@@ -11,13 +11,17 @@ b <- m$data %>>% rnorm %>>% as.character()
 nanotest(inherits(call_mirai(m), "mirai"))
 nanotest(m$data == 3L)
 nanotest(identical(call_mirai(m), m))
-nanotest(is_mirai(m) == TRUE)
+nanotest(is_mirai(m))
 nanotest(length(b$data) == 3L)
 nanotest(is.character(b$data))
-dm <- eval_mirai(as.matrix(df), df = data.frame(), .timeout = 2000)
-nanotest(inherits(call_mirai(dm), "mirai"))
-nanotest(unresolved(dm) == FALSE)
+df <- data.frame(a = 1, b = 2)
+dm <- eval_mirai(as.matrix(df), .args = list(df), .timeout = 2000)
+nanotest(is_mirai(call_mirai(dm)))
+nanotest(!unresolved(dm))
+nanotest(is.matrix(dm$data))
 nanotest(is.null(stop_mirai(dm)))
+me <- mirai(stop())
+nanotest(is_mirai_error(call_mirai(me)[["data"]]) && is_error_value(me$data))
 nanotest(daemons("view") == 1L)
 nanotest(daemons(0L) == -1L)
 nanotest(daemons("view") == 0L)
